unit Unit1;

interface
uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  StdCtrls, ComCtrls, ExtCtrls;

Type
  TFrmSpiral = class(TForm)
    GrBxControl: TGroupBox;
    RdGpSpirals: TRadioGroup;
    btDessiner: TButton;
    btQuitter: TButton;
    GrBxOptions: TGroupBox;
    Label2: TLabel;
    EdNbPI: TEdit;
    UpDownEndPI: TUpDown;
    Label1: TLabel;
    EdStartPI: TEdit;
    UpDownStartPI: TUpDown;
    Label3: TLabel;
    Label4: TLabel;
    LbA: TLabel;
    LbB: TLabel;
    EdA: TEdit;
    EdB: TEdit;
    UpDownA: TUpDown;
    UpDownB: TUpDown;
    procedure btQuitterClick(Sender: TObject);
    procedure RdGpSpiralsClick(Sender: TObject);
    procedure btDessinerClick(Sender: TObject);
    procedure UpDownAClick(Sender: TObject; Button: TUDBtnType);
    procedure UpDownBClick(Sender: TObject; Button: TUDBtnType);
    procedure FormCreate(Sender: TObject);
  End;

Const
     ARCHIMEDES    = 0;   {Spirale d'archimedes}
     LOGARITHME    = 1;   {Spirale Logarithmique}
     FERMATS       = 2;   {Spirale de Fermats}
     HYPERBOLIQUE  = 3;   {Spirale Hyperbolique}
     LITUUS        = 4;   {Spirale de Lituus}
     PARABOLIQUE   = 5;   {Siprale Parabolique}

Procedure ArchimedesSpiral( Const StartPI :Integer; Const EndPI :Integer );   {Spirale d'archimedes}
Procedure LogaritmiqueSpiral( Const StartPI :Integer; Const EndPI :Integer ); {Spirale Logarithmique}
Procedure FermatsSpiral( Const StartPI :Integer; Const EndPI :Integer );      {Spirale de Fermats}
Procedure HyperbolicSpiral( Const StartPI :Integer; Const EndPI :Integer );   {Spirale Hyperbolique}
Procedure LituusSpiral( Const StartPI :Integer; Const EndPI :Integer );       {Spirale de Lituus}
Procedure ParabolicSpiral( Const StartPI :Integer; Const EndPI :Integer );    {Siprale Parabolique}

Var
  FrmSpiral: TFrmSpiral;
  Increment :Single;       {Incrmentation pour les UpDown des facteurs A et B}

implementation
{$R *.DFM}


Procedure ArchimedesSpiral( Const StartPI :Integer; Const EndPI :Integer );
{Objectif :Dessiner une Spirale d'archimedes.
           Voici la formule :
                              Rayon = A( Theta ) }
Var
   I :Single;            {Degrs}
   X, Y :Single;         {Coordonnes}
   A :Single;            {Paramtre #1}
   DimX, DimY :Integer;  {Dimension de la fentre}
   Rayon :Single;        {Rayon}   

Begin
     DimX := FrmSpiral.Width Div 2;
     DimY := (FrmSpiral.Height - FrmSpiral.GrBxControl.Height) Div 2;
     I := StartPI * PI;

     A := StrToFloat(FrmSpiral.EdA.Text);

     While (I <= EndPI * PI) Do
     Begin
          Rayon := A * I;
          X := Rayon * Cos(I);
          Y := Rayon * Sin(I);

          FrmSpiral.Canvas.Pixels[ Round( (DimX + X) ),
                                   Round( (DimY - Y) ) ] := clRed;
          I := I + 0.05;
     End;
End;

Procedure LogaritmiqueSpiral( Const StartPI :Integer; Const EndPI :Integer );
{Objectif :Dessiner une Spirale Equiangulaire.
           Voici la formule :
                              Rayon = A Exp(B Theta) }
Var
   I :Single;            {Degrs}
   X, Y :Single;         {Coordonnes}
   DimX, DimY :Integer;  {Dimension de la fentre}
   A :Single;            {Paramtre #1}
   B :Single;            {Paramtre #2}
   Rayon :Single;        {Rayon}

Begin
     DimX := FrmSpiral.Width Div 2;
     DimY := (FrmSpiral.Height - FrmSpiral.GrBxControl.Height) Div 2;
     I := StartPI * PI;

     A := StrToFloat(FrmSpiral.EdA.Text);
     B := StrToFloat(FrmSpiral.EdB.Text);

     While (I <= EndPI * PI) Do
     Begin
          Rayon := A * Exp( B * I );

          X := Rayon * Cos(I);
          Y := Rayon * Sin(I);
          FrmSpiral.Canvas.Pixels[ Round( (DimX + X) ),
                                   Round( (DimY - Y) ) ] := clRed;
          I := I + 0.05;
     End;
End;


Procedure FermatsSpiral( Const StartPI :Integer; Const EndPI :Integer );
{Objectif :Dessiner une Spirale de Fermats.
           Voici la formule :
                              Rayon = Sqtr(Theta) }

Var
   I :Single;            {Degrs}
   X, Y :Single;         {Coordonnes}
   DimX, DimY :Integer;  {Dimension de la fentre}
   Rayon :Single;        {Rayon}

Begin
     DimX := FrmSpiral.Width Div 2;
     DimY := (FrmSpiral.Height - FrmSpiral.GrBxControl.Height) Div 2;
     I := StartPI * PI;

     While (I <= EndPI * PI) Do
     Begin
          Rayon :=  20 * Sqrt( I );

          X := Rayon * Cos(I);
          Y := Rayon * Sin(I);
          FrmSpiral.Canvas.Pixels[ Round( (DimX + X) ),
                                   Round( (DimY - Y) ) ] := clRed;
          I := I + 0.05;
     End;
End;


Procedure HyperbolicSpiral( Const StartPI :Integer; Const EndPI :Integer );
{Objectif :Dessiner une Spirale Hyperbolique.
           Voici la formule :
                              Rayon = A / Theta }

Var
   I :Single;            {Degrs}
   X, Y :Single;         {Coordonnes}
   DimX, DimY :Integer;  {Dimension de la fentre}
   Rayon :Single;        {Rayon}
   A :Single;            {Paramtre #1}

Begin
     DimX := FrmSpiral.Width Div 2;
     DimY := (FrmSpiral.Height - FrmSpiral.GrBxControl.Height) Div 2;

     A := StrToFloat(FrmSpiral.EdA.Text);
     I := (StartPI * PI) + 0.001;

     While (I <= EndPI * PI) Do
     Begin
          Rayon :=  A / I;

          X := Rayon * Cos(I);
          Y := Rayon * Sin(I);
          FrmSpiral.Canvas.Pixels[ Round( (DimX + X) ),
                                   Round( (DimY - Y) ) ] := clRed;
          I := I + 0.05;
     End;
End;

Procedure LituusSpiral( Const StartPI :Integer; Const EndPI :Integer );
{Objectif :Dessiner une Spirale de Lituus.
           Voici la formule :
                              Rayon^2 = A^2 / Theta }

Var
   I :Single;            {Degrs}
   X, Y :Single;         {Coordonnes}
   DimX, DimY :Integer;  {Dimension de la fentre}
   Rayon :Single;        {Rayon}
   A :Single;            {Paramtre #1}

Begin
     DimX := FrmSpiral.Width Div 2;
     DimY := (FrmSpiral.Height - FrmSpiral.GrBxControl.Height) Div 2;

     A := StrToFloat(FrmSpiral.EdA.Text);
     I := (StartPI * PI) + 0.001;

     While (I <= EndPI * PI) Do
     Begin
          Rayon := Sqrt(Sqr(A) / I);
          Rayon := Sqr(Rayon);

          X := Rayon * Cos(I);
          Y := Rayon * Sin(I);
          FrmSpiral.Canvas.Pixels[ Round( (DimX + X) ),
                                   Round( (DimY - Y) ) ] := clRed;
          I := I + 0.05;
     End;
End;

Procedure ParabolicSpiral( Const StartPI :Integer; Const EndPI :Integer );
{Objectif :Dessiner une Siprale Parabolique.
           Voici la formule :
                              (Radius - A)^2 = B^2 Theta }

Var
   I :Single;            {Degrs}
   X, Y :Single;         {Coordonnes}
   DimX, DimY :Integer;  {Dimension de la fentre}
   Rayon :Single;        {Rayon}
   A :Single;            {Paramtre #1 d'largissement}
   B :Single;            {Paramtre #2 d'largissement}

Begin
     DimX := FrmSpiral.Width Div 2;
     DimY := (FrmSpiral.Height - FrmSpiral.GrBxControl.Height) Div 2;

     A := StrToFloat(FrmSpiral.EdA.Text);
     B := StrToFloat(FrmSpiral.EdB.Text);
     I := StartPI * PI;

     While (I <= EndPI * PI) Do
     Begin
          Rayon := (B * B) * I;

          X := (Sqr(Rayon - A)) * Cos(I);
          Y := (Sqr(Rayon - A)) * Sin(I);
          FrmSpiral.Canvas.Pixels[ Round( (DimX + X) ),
                                   Round( (DimY - Y) ) ] := clRed;
          I := I + 0.05;
     End;
End;

procedure TFrmSpiral.RdGpSpiralsClick(Sender: TObject);
{Objectif :Dlimiter le nombre de cycle  effectuer selon la spiral choisie.}
begin
     Case RdGpSpirals.ItemIndex Of
           ARCHIMEDES:Begin
                           UpDownStartPI.Min := -50;
                           UpDownStartPI.Max := 0;
                           UpDownEndPI.Min   := 0;
                           UpDownEndPI.Max   := 50;
                           FrmSpiral.Caption := 'Rayon = A( Theta )';
                           Increment := 1;
                           EdA.Visible := True;
                           UpDownA.Visible := True;
                           EdB.Visible := False;
                           UpDownB.Visible := False;
                           UpDownA.Min :=0;
                           UpDownA.Max :=300;
                           UpDownA.Position := Round(Increment * UpDownA.Min);
                           EdA.Text := FloatToStr(UpDownA.Position);
                      End;
           LOGARITHME:Begin
                           UpDownStartPI.Min := -20;
                           UpDownStartPI.Max := 0;
                           UpDownEndPI.Min   := 0;
                           UpDownEndPI.Max   := 20;
                           FrmSpiral.Caption := 'Rayon = A Exp(B Theta)';
                           EdA.Visible := True;
                           UpDownA.Visible := True;
                           EdB.Visible := True;
                           UpDownB.Visible := True;
                           Increment := 0.05;
                           UpDownA.Min :=0;
                           UpDownA.Max :=100;
                           UpDownB.Min :=0;
                           UpDownB.Max :=20;
                           UpDownA.Position := Round(Increment * UpDownA.Min);
                           UpDownB.Position := Round(Increment * UpDownB.Min);
                           EdA.Text := FloatToStr(UpDownA.Position);
                           EdB.Text := FloatToStr(UpDownB.Position);
                      End;
           FERMATS:Begin
                        UpDownStartPI.Min := 0;
                        UpDownStartPI.Max := 0;
                        UpDownEndPI.Min   := 0;
                        UpDownEndPI.Max   := 30;
                        FrmSpiral.Caption := 'Rayon = Sqtr(Theta)';
                        EdA.Visible := False;
                        UpDownA.Visible := False;
                        EdB.Visible := False;
                        UpDownB.Visible := False;
                   End;
           HYPERBOLIQUE:Begin
                             UpDownStartPI.Min := -20;
                             UpDownStartPI.Max := 0;
                             UpDownEndPI.Min   := 0;
                             UpDownEndPI.Max   := 20;
                             FrmSpiral.Caption := 'Rayon = A / Theta';
                             EdA.Visible := True;
                             UpDownA.Visible := True;
                             EdB.Visible := False;
                             UpDownB.Visible := False;
                             UpDownA.Min :=0;
                             UpDownA.Max :=200;
                             Increment := 1;
                             UpDownA.Position := Round(Increment * UpDownA.Min);
                             EdA.Text := FloatToStr(UpDownA.Position);
                        End;
            LITUUS:Begin
                        UpDownStartPI.Min := -20;
                        UpDownStartPI.Max := 0;
                        UpDownEndPI.Min   := 0;
                        UpDownEndPI.Max   := 20;
                        FrmSpiral.Caption := 'Rayon = A Exp(B Theta)';
                        EdA.Visible := True;
                        UpDownA.Visible := True;
                        EdB.Visible := False;
                        UpDownB.Visible := False;
                        Increment := 0.05;
                        UpDownA.Min :=0;
                        UpDownA.Max :=120;
                        UpDownA.Position := Round(Increment * UpDownA.Min);
                        EdA.Text := FloatToStr(UpDownA.Position);
                   End;
            PARABOLIQUE:Begin
                             UpDownStartPI.Min := -20;
                             UpDownStartPI.Max := 0;
                             UpDownEndPI.Min   := 0;
                             UpDownEndPI.Max   := 20;
                             FrmSpiral.Caption := 'Rayon^2 = A^2 / Theta';
                             EdA.Visible := True;
                             UpDownA.Visible := True;
                             EdB.Visible := True;
                             UpDownB.Visible := True;
                             Increment := 0.05;
                             UpDownA.Min :=0;
                             UpDownA.Max :=300;
                             UpDownB.Min :=0;
                             UpDownB.Max :=20;
                             UpDownA.Position := Round(Increment * UpDownA.Min);
                             UpDownB.Position := Round(Increment * UpDownB.Min);
                             EdA.Text := FloatToStr(UpDownA.Position);
                             EdB.Text := FloatToStr(UpDownB.Position);
                        End;
          End;
          UpDownStartPI.Position   := UpDownStartPI.Max;
          UpDownEndPI.Position   := UpDownEndPI.Min;
end;

procedure TFrmSpiral.btDessinerClick(Sender: TObject);
{Objectif :Effacer la zone.
           Dessiner la spiral choisie par l'utilisateur}
Begin
     If (UpDownEndPI.Position - UpDownStartPI.Position) > 0 Then
     Begin
          FrmSpiral.Canvas.Brush.Color := clBlack;
          FrmSpiral.Canvas.FillRect( Rect(0, 0, Width, Height) );

          Case RdGpSpirals.ItemIndex Of
              ARCHIMEDES:Begin
                              ArchimedesSpiral( UpDownStartPI.Position, UpDownEndPI.Position );
                         End;
              LOGARITHME:Begin
                              LogaritmiqueSpiral( UpDownStartPI.Position, UpDownEndPI.Position );
                         End;
              FERMATS:Begin
                           FermatsSpiral( UpDownStartPI.Position, UpDownEndPI.Position );
                      End;
              HYPERBOLIQUE:Begin
                                HyperbolicSpiral( UpDownStartPI.Position, UpDownEndPI.Position );
                           End;
              LITUUS:Begin
                          LituusSpiral( UpDownStartPI.Position, UpDownEndPI.Position );
                     End;
              PARABOLIQUE:Begin
                               ParabolicSpiral( UpDownStartPI.Position, UpDownEndPI.Position );
                          End;
          End;
     End
     Else
         ShowMessage(' Erreur la position de dmarrage de PI est plus lev ' +
                     'que celle de fin ');
End;

procedure TFrmSpiral.btQuitterClick(Sender: TObject);
{Objectif :Fermer l'application}
begin
     Close;
end;
procedure TFrmSpiral.UpDownAClick(Sender: TObject; Button: TUDBtnType);
{Objectif :Affecter des valeurs aux paramtres A et B}
begin
     EdA.Text :=  FloatToStr( UpDownA.Position * Increment );
end;

procedure TFrmSpiral.UpDownBClick(Sender: TObject; Button: TUDBtnType);
{Objectif :Affecter des valeurs aux paramtres A et B}
begin
     EdB.Text :=  FloatToStr( UpDownB.Position * Increment );
end;

procedure TFrmSpiral.FormCreate(Sender: TObject);
{Objectif :Valeur d'incrmentation par dfaut au dbut, donc pour la spiral
           choisi au dpart [Archimedes]}
begin
     Increment := 1;
end;

End.

